//+---------------------------------------------------------------------+
//|                                            ColorBlauErgodicMACD.mq5 |
//|                                  Copyright  2013, Nikolay Kositsin | 
//|                                 Khabarovsk,   farria@mail.redcom.ru | 
//+---------------------------------------------------------------------+ 
//|     SmoothAlgorithms.mqh      |
//|   (): __\\MQL5\Include        |
//+---------------------------------------------------------------------+
#property copyright "Copyright  2013, Nikolay Kositsin"
#property link "farria@mail.redcom.ru" 
//---   
#property version   "1.00"
//---     
#property indicator_separate_window 
//---    4
#property indicator_buffers 4 
//---     
#property indicator_plots   2
//+-----------------------------------+
//|       |
//+-----------------------------------+
//---      
#property indicator_type1 DRAW_COLOR_HISTOGRAM
//---      
#property indicator_color1 clrMediumVioletRed,clrDarkOrange,clrGray,clrCornflowerBlue,clrMediumBlue
//---   - 
#property indicator_style1 STYLE_SOLID
//---     2
#property indicator_width1 2
//---   
#property indicator_label1 "Blau Ergodic MACD"

//---      
#property indicator_type2 DRAW_COLOR_LINE
//---      
#property indicator_color2 clrMagenta,clrGray,clrLimeGreen
//---   -   
#property indicator_style2 STYLE_SOLID
//---     4
#property indicator_width2 4
//---    
#property indicator_label2  "Signal Line"
//+-----------------------------------+
//|          |
//+-----------------------------------+
#include <SmoothAlgorithms.mqh> 
//+-----------------------------------+

//---    CXMA   SmoothAlgorithms.mqh
CXMA XMA1,XMA2,XMA3,XMA4;
//+-----------------------------------+
//|             |
//+-----------------------------------+
enum Applied_price_ // 
  {
   PRICE_CLOSE_ = 1,     //Close
   PRICE_OPEN_,          //Open
   PRICE_HIGH_,          //High
   PRICE_LOW_,           //Low
   PRICE_MEDIAN_,        //Median Price (HL/2)
   PRICE_TYPICAL_,       //Typical Price (HLC/3)
   PRICE_WEIGHTED_,      //Weighted Close (HLCC/4)
   PRICE_SIMPL_,         //Simpl Price (OC/2)
   PRICE_QUARTER_,       //Quarted Price (HLOC/4) 
   PRICE_TRENDFOLLOW0_,  //TrendFollow_1 Price 
   PRICE_TRENDFOLLOW1_,  //TrendFollow_2 Price 
   PRICE_DEMARK_         //Demark Price 
  };
/*enum Smooth_Method -     SmoothAlgorithms.mqh
  {
   MODE_SMA_,  //SMA
   MODE_EMA_,  //EMA
   MODE_SMMA_, //SMMA
   MODE_LWMA_, //LWMA
   MODE_JJMA,  //JJMA
   MODE_JurX,  //JurX
   MODE_ParMA, //ParMA
   MODE_T3,    //T3
   MODE_VIDYA, //VIDYA
   MODE_AMA,   //AMA
  }; */
//+-----------------------------------+
//|                   |
//+-----------------------------------+
input Smooth_Method XMA_Method=MODE_EMA;        //   
input int Fast_XMA=5;                           //   
input int Slow_XMA = 20;                        //   
input int XPhase = 100;                         //   
//--- XPhase:  JJMA    -100 ... +100,     ;
//--- XPhase:  VIDIA   CMO,  AMA    
input int MACD_XMA=3;                           //    MACD
input Smooth_Method Signal_Method=MODE_EMA;     //    
input int Signal_XMA=3;                         //    
input int Signal_Phase=100;                     //   
input Applied_price_ AppliedPrice=PRICE_CLOSE_; //  
//---      
int min_rates_total,min_rates_1,min_rates_2;
//---   ,   
//---      
double XMACDBuffer[],SignBuffer[],ColorXMACDBuffer[],ColorSignBuffer[];
//+------------------------------------------------------------------+    
//| XMACD indicator initialization function                          | 
//+------------------------------------------------------------------+  
void OnInit()
  {
//---     
   min_rates_1=MathMax(XMA1.GetStartBars(XMA_Method,Fast_XMA,XPhase),XMA1.GetStartBars(XMA_Method,Slow_XMA,XPhase));
   min_rates_2=min_rates_1+XMA1.GetStartBars(XMA_Method,MACD_XMA,XPhase);
   min_rates_total=min_rates_2+XMA1.GetStartBars(Signal_Method,Signal_XMA,Signal_Phase)+2;

//---    XMACDBuffer   
   SetIndexBuffer(0,XMACDBuffer,INDICATOR_DATA);

//---          
   SetIndexBuffer(1,ColorXMACDBuffer,INDICATOR_COLOR_INDEX);
//---    SignBuffer   
   SetIndexBuffer(2,SignBuffer,INDICATOR_DATA);
//---          
   SetIndexBuffer(3,ColorSignBuffer,INDICATOR_COLOR_INDEX);

//---      
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,min_rates_total);
//---   ,      
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,EMPTY_VALUE);

//---      
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,min_rates_total);
//---   ,      
   PlotIndexSetDouble(1,PLOT_EMPTY_VALUE,EMPTY_VALUE);

//---       
   XMA1.XMALengthCheck("Fast_XMA", Fast_XMA);
   XMA1.XMALengthCheck("Slow_XMA", Slow_XMA);
   XMA1.XMALengthCheck("MACD_XMA", MACD_XMA);
   XMA1.XMALengthCheck("Signal_XMA", Signal_XMA);
//---       
   XMA1.XMAPhaseCheck("XPhase", XPhase, XMA_Method);
   XMA1.XMAPhaseCheck("Signal_Phase", Signal_Phase, Signal_Method);

//---      
   string shortname;
   string Smooth1=XMA1.GetString_MA_Method(XMA_Method);
   string Smooth2=XMA1.GetString_MA_Method(Signal_Method);
   StringConcatenate(shortname,
                     "ColorBlauErgodicMACD( ",Fast_XMA,", ",Slow_XMA,", ",MACD_XMA,", ",Signal_XMA,", ",Smooth1,", ",Smooth2," )");
//---           
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);
//---     
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits+1);
//---  
  }
//+------------------------------------------------------------------+  
//| XMACD iteration function                                         | 
//+------------------------------------------------------------------+  
int OnCalculate(const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---       
   if(rates_total<min_rates_total) return(0);

//---   
   int first,clr,bar;
//---       
   double price_,fast_xma,slow_xma,xmacd,xxmacd,sign;

//---     OnCalculate()
   if(prev_calculated>rates_total || prev_calculated<=0)//      
     {
      first=0; //        
     }
   else        //      
     {
      first=prev_calculated-1;
     }

//---    
   for(bar=first; bar<rates_total && !IsStopped(); bar++)
     {
      price_=PriceSeries(AppliedPrice,bar,open,low,high,close);

      fast_xma=XMA1.XMASeries(0,prev_calculated,rates_total,XMA_Method,XPhase,Fast_XMA,price_,bar,false);
      slow_xma=XMA2.XMASeries(0,prev_calculated,rates_total,XMA_Method,XPhase,Slow_XMA,price_,bar,false);

      xmacd=fast_xma-slow_xma;
      xxmacd=XMA3.XMASeries(min_rates_1,prev_calculated,rates_total,XMA_Method,XPhase,MACD_XMA,xmacd,bar,false);
      sign=XMA4.XMASeries(min_rates_2,prev_calculated,rates_total,Signal_Method,Signal_Phase,Signal_XMA,xxmacd,bar,false);

      //---            
      XMACDBuffer[bar]=xxmacd;
      SignBuffer[bar]=sign;
     }

   if(prev_calculated>rates_total || prev_calculated<=0) first=min_rates_total;
//---    
   for(bar=first; bar<rates_total && !IsStopped(); bar++)
     {
      clr=2;

      if(XMACDBuffer[bar]>0)
        {
         if(XMACDBuffer[bar]>XMACDBuffer[bar-1]) clr=4;
         if(XMACDBuffer[bar]<XMACDBuffer[bar-1]) clr=3;
        }

      if(XMACDBuffer[bar]<0)
        {
         if(XMACDBuffer[bar]<XMACDBuffer[bar-1]) clr=0;
         if(XMACDBuffer[bar]>XMACDBuffer[bar-1]) clr=1;
        }

      ColorXMACDBuffer[bar]=clr;
     }

//---     
   for(bar=first; bar<rates_total && !IsStopped(); bar++)
     {
      clr=1;
      if(XMACDBuffer[bar]>SignBuffer[bar-1]) clr=2;
      if(XMACDBuffer[bar]<SignBuffer[bar-1]) clr=0;
      ColorSignBuffer[bar]=clr;
     }
//---     
   return(rates_total);
  }
//+------------------------------------------------------------------+
